"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.domGenerators = exports.OnNavigationEventGenerator = exports.OnChangeGenerator = exports.FormSubmitGenerator = exports.ClickSignalsGenerator = exports.cleanText = void 0;
const detect_url_change_1 = require("../../lib/detect-url-change");
const logger_1 = require("../../lib/logger");
const factories_1 = require("../../types/factories");
const parseFormData = (data) => {
    return [...data].reduce((acc, [key, value]) => {
        if (typeof value === 'string') {
            acc[key] = value;
        }
        return acc;
    }, {});
};
const parseLabels = (labels) => {
    if (!labels)
        return [];
    return [...labels]
        .map((label) => ({
        id: label.id,
        attributes: parseNodeMap(label.attributes),
        textContent: label.textContent ? (0, exports.cleanText)(label.textContent) : undefined,
    }))
        .filter((el) => Boolean(el.textContent));
};
const parseNodeMap = (nodeMap) => {
    return Array.from(nodeMap).reduce((acc, attr) => {
        acc[attr.name] = attr.value;
        return acc;
    }, {});
};
const cleanText = (str) => {
    return str
        .replace(/[\r\n\t]+/g, ' ') // Replace newlines and tabs with a space
        .replace(/\s\s+/g, ' ') // Replace multiple spaces with a single space
        .replace(/\u00A0/g, ' ') // Replace non-breaking spaces with a regular space
        .trim(); // Trim leading and trailing spaces
};
exports.cleanText = cleanText;
const parseElement = (el) => {
    const labels = parseLabels(el.labels);
    const base = {
        // adding a bunch of fields that are not on _all_ elements, but are on enough that it's useful to have them here.
        attributes: parseNodeMap(el.attributes),
        classList: [...el.classList],
        id: el.id,
        labels,
        label: labels[0],
        name: el.name,
        nodeName: el.nodeName,
        tagName: el.tagName,
        title: el.title,
        type: el.type,
        value: el.value,
        textContent: (el.textContent && (0, exports.cleanText)(el.textContent)) ?? undefined,
        innerText: (el.innerText && (0, exports.cleanText)(el.innerText)) ?? undefined,
    };
    if (el instanceof HTMLSelectElement) {
        return {
            ...base,
            selectedOptions: [...el.selectedOptions].map((option) => ({
                value: option.value,
                text: option.text,
            })),
            selectedIndex: el.selectedIndex,
        };
    }
    else if (el instanceof HTMLInputElement) {
        return {
            ...base,
            checked: el.checked,
        };
    }
    else if (el instanceof HTMLMediaElement) {
        return {
            ...base,
            currentSrc: el.currentSrc,
            currentTime: el.currentTime,
            duration: el.duration,
            ended: el.ended,
            muted: el.muted,
            paused: el.paused,
            playbackRate: el.playbackRate,
            readyState: el.readyState,
            src: el.src,
            volume: el.volume,
        };
    }
    else if (el instanceof HTMLFormElement) {
        return {
            ...base,
            innerText: undefined,
            textContent: undefined,
            formData: parseFormData(new FormData(el)),
        };
    }
    return base;
};
class ClickSignalsGenerator {
    constructor() {
        this.id = 'click';
    }
    register(emitter) {
        const handleClick = (ev) => {
            const target = ev.target;
            if (!target)
                return;
            const el = this.getClosestClickableElement(target);
            if (el) {
                emitter.emit((0, factories_1.createInteractionSignal)({
                    eventType: 'click',
                    target: parseElement(el),
                }));
            }
        };
        document.addEventListener('click', handleClick, true);
        return () => document.removeEventListener('click', handleClick);
    }
    getClosestClickableElement(el) {
        // if you click on a nested element, we want to get the closest clickable ancestor. Useful for things like buttons with nested text or images
        return el.closest('button, a, [role="button"], [role="link"]');
    }
}
exports.ClickSignalsGenerator = ClickSignalsGenerator;
class FormSubmitGenerator {
    constructor() {
        this.id = 'form-submit';
    }
    register(emitter) {
        const handleSubmit = (ev) => {
            const target = ev.target;
            if (!target)
                return;
            // reference to the form element that the submit event is being fired at
            const submitter = ev.submitter;
            // If the form is submitted via JavaScript using form.submit(), the submitter property will be null because no specific button/input triggered the submission.
            emitter.emit((0, factories_1.createInteractionSignal)({
                eventType: 'submit',
                target: parseElement(target),
                submitter: submitter ? parseElement(submitter) : undefined,
            }));
        };
        document.addEventListener('submit', handleSubmit, true);
        return () => document.removeEventListener('submit', handleSubmit);
    }
}
exports.FormSubmitGenerator = FormSubmitGenerator;
class OnChangeGenerator {
    constructor() {
        this.id = 'change';
    }
    register(emitter) {
        const handleChange = (ev) => {
            const target = ev.target;
            if (!target)
                return;
            if (target && target instanceof HTMLInputElement) {
                if (target.type === 'password') {
                    logger_1.logger.debug('Ignoring change event for input', target);
                    return;
                }
            }
            emitter.emit((0, factories_1.createInteractionSignal)({
                eventType: 'change',
                target: parseElement(target),
            }));
        };
        document.addEventListener('change', handleChange, true);
        return () => document.removeEventListener('change', handleChange);
    }
}
exports.OnChangeGenerator = OnChangeGenerator;
class OnNavigationEventGenerator {
    constructor() {
        this.id = 'navigation';
    }
    register(emitter) {
        // emit navigation signal on page load
        emitter.emit((0, factories_1.createNavigationSignal)({
            action: 'pageLoad',
            ...this.createCommonFields(),
        }));
        // emit a navigation signal whenever the URL has changed
        const urlChange = new detect_url_change_1.URLChangeObservable();
        urlChange.subscribe((prevUrl) => emitter.emit((0, factories_1.createNavigationSignal)({
            action: 'urlChange',
            prevUrl,
            ...this.createCommonFields(),
        })));
        return () => {
            urlChange.unsubscribe();
        };
    }
    createCommonFields() {
        return {
            // these fields are named after those from the page call, rather than a DOM api.
            url: location.href,
            path: location.pathname,
            hash: location.hash,
            search: location.search,
            title: document.title,
        };
    }
}
exports.OnNavigationEventGenerator = OnNavigationEventGenerator;
exports.domGenerators = [
    ClickSignalsGenerator,
    FormSubmitGenerator,
    OnChangeGenerator,
    OnNavigationEventGenerator,
];
//# sourceMappingURL=dom-gen.js.map