"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.Signals = void 0;
const client_1 = require("../client");
const buffer_1 = require("../buffer");
const emitter_1 = require("../emitter");
const dom_gen_1 = require("../signal-generators/dom-gen");
const network_gen_1 = require("../signal-generators/network-gen");
const register_1 = require("../signal-generators/register");
const analytics_service_1 = require("../analytics-service");
const processor_1 = require("../processor/processor");
const sandbox_1 = require("../processor/sandbox");
const settings_1 = require("./settings");
const logger_1 = require("../../lib/logger");
class Signals {
    constructor(settingsConfig = {}) {
        this.preStartBuffer = [];
        this.cleanup = [];
        this.addToPreStartBuffer = (signal) => {
            this.preStartBuffer.push(signal);
        };
        /**
         * Flush/process any signals that were emitted before the start method was called.
         */
        this.flushPreStartBuffer = (processor) => {
            logger_1.logger.debug(`Flushing ${this.preStartBuffer.length} events in pre-start buffer`, this.preStartBuffer);
            this.signalEmitter.unsubscribe(this.addToPreStartBuffer);
            this.preStartBuffer.forEach(async (signal) => {
                void processor.process(signal, await this.buffer.getAll());
            });
            this.preStartBuffer = [];
        };
        this.globalSettings = new settings_1.SignalGlobalSettings(settingsConfig);
        this.signalEmitter = new emitter_1.SignalEmitter();
        this.signalsClient = new client_1.SignalsIngestClient(this.globalSettings.ingestClient);
        this.buffer = (0, buffer_1.getSignalBuffer)(this.globalSettings.signalBuffer);
        this.signalEmitter.subscribe((signal) => {
            void this.signalsClient.send(signal);
            void this.buffer.add(signal);
        });
        this.signalEmitter.subscribe(this.addToPreStartBuffer);
        void this.registerGenerator([
            ...dom_gen_1.domGenerators,
            new network_gen_1.NetworkGenerator(this.globalSettings.network),
        ]);
    }
    /**
     * Does the following:
     * - Sends any queued signals to the server.
     * - Augments the analytics client to transform events -> signals
     * - Registers custom signal generators.
     */
    async start(analytics) {
        const analyticsService = new analytics_service_1.AnalyticsService(analytics);
        this.globalSettings.update({
            edgeFnDownloadURL: analyticsService.edgeFnSettings?.downloadURL,
            disallowListURLs: [
                analyticsService.instance.settings.apiHost,
                analyticsService.instance.settings.cdnURL,
            ],
            sampleRate: analyticsService.instance.settings.cdnSettings
                .autoInstrumentationSettings?.sampleRate ?? 0,
        });
        const sandbox = new sandbox_1.Sandbox(new sandbox_1.SandboxSettings(this.globalSettings.sandbox));
        const processor = new processor_1.SignalEventProcessor(analyticsService.instance, sandbox);
        void this.flushPreStartBuffer(processor);
        this.signalEmitter.subscribe(async (signal) => {
            void processor.process(signal, await this.buffer.getAll());
        });
        await this.registerGenerator([
            analyticsService.createSegmentInstrumentationEventGenerator(),
        ]);
        await this.signalsClient.init({
            writeKey: analyticsService.instance.settings.writeKey,
        });
    }
    stop() {
        this.cleanup.forEach((fn) => fn());
    }
    clearStorage() {
        void this.buffer.clear();
    }
    /**
     * Emit custom signals.
     */
    async registerGenerator(generators) {
        this.cleanup.push(await (0, register_1.registerGenerator)(this.signalEmitter, generators));
    }
}
exports.Signals = Signals;
//# sourceMappingURL=signals.js.map