import { openDB } from 'idb';
import { logger } from '../../lib/logger';
export class SignalStore {
    length() {
        return this.signalCount;
    }
    static deleteDatabase() {
        return indexedDB.deleteDatabase(SignalStore.DB_NAME);
    }
    constructor(settings = {}) {
        this.signalCount = 0;
        this.maxBufferSize = settings.maxBufferSize ?? 50;
        this.signalStore = this.createSignalStore();
        void this.initializeSignalCount();
    }
    getStore() {
        return this.signalStore;
    }
    async createSignalStore() {
        const db = await openDB(SignalStore.DB_NAME, 1, {
            upgrade(db) {
                db.createObjectStore(SignalStore.STORE_NAME, { autoIncrement: true });
            },
        });
        logger.debug('Signals Buffer (indexDB) initialized');
        return db;
    }
    async initializeSignalCount() {
        const store = await this.signalStore;
        this.signalCount = await store.count(SignalStore.STORE_NAME);
        logger.debug(`Signal count initialized with ${this.signalCount} signals (max: ${this.maxBufferSize})`);
    }
    async add(signal) {
        const store = await this.signalStore;
        if (this.signalCount >= this.maxBufferSize) {
            // Get the key of the oldest signal and delete it
            const oldestKey = await store
                .transaction(SignalStore.STORE_NAME)
                .store.getKey(IDBKeyRange.lowerBound(0));
            if (oldestKey !== undefined) {
                await store.delete(SignalStore.STORE_NAME, oldestKey);
            }
            else {
                this.signalCount--;
            }
        }
        await store.add(SignalStore.STORE_NAME, signal);
        this.signalCount++;
    }
    /**
     * Get list of signals from the store, with the newest signals first.
     */
    async getAll() {
        const store = await this.getStore();
        return (await store.getAll(SignalStore.STORE_NAME)).reverse();
    }
    async clear() {
        const store = await this.getStore();
        return store.clear(SignalStore.STORE_NAME);
    }
}
SignalStore.DB_NAME = 'Segment Signals Buffer';
SignalStore.STORE_NAME = 'signals';
export class SignalBuffer {
    constructor(store) {
        this.store = store;
    }
    async add(signal) {
        try {
            return await this.store.add(signal);
        }
        catch (e) {
            console.error(e);
            return undefined;
        }
    }
    async getAll() {
        try {
            return await this.store.getAll();
        }
        catch (e) {
            console.error(e);
            return [];
        }
    }
    async clear() {
        try {
            return await this.store.clear();
        }
        catch (e) {
            console.error(e);
            return undefined;
        }
    }
}
export const getSignalBuffer = (settings) => {
    const store = settings.signalStorage ?? new SignalStore(settings);
    return new SignalBuffer(store);
};
//# sourceMappingURL=index.js.map