import { Analytics, segmentio } from '@segment/analytics-next';
import { logger } from '../../lib/logger';
import { redactSignalData } from './redact';
export class SignalsIngestSettings {
    constructor(settings) {
        this.flushAt = settings.flushAt ?? 5;
        this.apiHost = settings.apiHost ?? 'signals.segment.io/v1';
        this.flushInterval = settings.flushInterval ?? 2000;
        this.shouldDisableSignalsRedaction =
            settings.shouldDisableSignalsRedaction ?? (() => false);
        this.shouldIngestSignals = settings.shouldIngestSignals ?? (() => false);
    }
}
/**
 * This currently just uses the Segment analytics-next library to send signals.
 * This persists the signals in a queue until the client is initialized.
 */
export class SignalsIngestClient {
    constructor(settings = {}) {
        /**
         * This matters to sort the signals in the UI if the timestamp conflict (which can happen very very rarely)
         */
        this.index = 0;
        this.settings = new SignalsIngestSettings(settings);
        this.buffer = [];
        this.analytics = undefined;
    }
    async createAnalyticsClient(settings) {
        const analytics = new Analytics({ writeKey: settings.writeKey });
        this.settings.writeKey = analytics.settings.writeKey;
        await analytics.register(segmentio(analytics, {
            apiHost: this.settings.apiHost,
            apiKey: this.settings.writeKey,
            deliveryStrategy: {
                config: {
                    size: this.settings.flushAt,
                    timeout: this.settings.flushInterval,
                },
                strategy: 'batching',
            },
        }));
        analytics.emit('initialize', settings);
        analytics.on('track', (...args) => {
            logger.debug('Track event from analytics client', ...args);
        });
        return analytics;
    }
    sendTrackCall(signal) {
        if (!this.analytics) {
            throw new Error('Please initialize before calling this method.');
        }
        if (!this.settings.shouldIngestSignals()) {
            return;
        }
        const disableRedaction = this.settings.shouldDisableSignalsRedaction();
        const cleanSignal = disableRedaction ? signal : redactSignalData(signal);
        if (disableRedaction) {
            logger.debug('Sending unredacted data to segment', cleanSignal);
        }
        const MAGIC_EVENT_NAME = 'Segment Signal Generated';
        return this.analytics.track(MAGIC_EVENT_NAME, {
            index: this.index++,
            type: signal.type,
            data: cleanSignal.data,
        });
    }
    /**
     *  Initialize analytics and flush any events queue.
     */
    async init({ writeKey }) {
        this.analytics = await this.createAnalyticsClient({ writeKey });
        this.flush();
        logger.debug('Init signals-analytics client', { writeKey });
    }
    send(signal) {
        if (!this.analytics) {
            logger.debug('Buffering signal', signal);
            this.buffer.push(signal);
        }
        else {
            return this.sendTrackCall(signal);
        }
    }
    flush() {
        if (!this.analytics) {
            throw new Error('Please initialize before calling this method.');
        }
        logger.debug('Flushing signals', this.buffer);
        this.buffer.forEach((signal) => {
            void this.sendTrackCall(signal);
        });
        this.buffer = [];
    }
}
//# sourceMappingURL=index.js.map