import { parseDebugModeQueryString } from '../debug-mode';
import { logger } from '../../lib/logger';
import { NetworkSettingsConfig } from '../signal-generators/network-gen';
/**
 * Global settings for the application
 * In the future this pattern may allows us to add settings which can be dynamically set by the user.
 * Currently, this is just a way to pass settings to the different parts of the application.
 */
export class SignalGlobalSettings {
    constructor(settings) {
        this.sampleSuccess = false;
        this.signalsDebug = new SignalsDebugSettings();
        if (settings.maxBufferSize && settings.signalStorage) {
            throw new Error('maxBufferSize and signalStorage cannot be defined at the same time');
        }
        this.signalsDebug = new SignalsDebugSettings(settings.disableSignalsRedaction, settings.enableSignalsIngestion);
        this.signalBuffer = {
            signalStorage: settings.signalStorage,
            maxBufferSize: settings.maxBufferSize,
        };
        this.ingestClient = {
            apiHost: settings.apiHost,
            flushAt: settings.flushAt,
            flushInterval: settings.flushInterval,
            shouldDisableSignalsRedaction: this.signalsDebug.getDisableSignalsRedaction,
            shouldIngestSignals: () => {
                if (this.signalsDebug.getEnableSignalsIngestion()) {
                    return true;
                }
                if (!this.sampleSuccess) {
                    return false;
                }
                return false;
            },
        };
        this.sandbox = {
            functionHost: settings.functionHost,
            processSignal: settings.processSignal,
            edgeFnDownloadURL: undefined,
        };
        this.network = new NetworkSettingsConfig({
            networkSignalsAllowList: settings.networkSignalsAllowList,
            networkSignalsDisallowList: settings.networkSignalsDisallowList,
            networkSignalsAllowSameDomain: settings.networkSignalsAllowSameDomain,
        });
    }
    update({ edgeFnDownloadURL, disallowListURLs, sampleRate, }) {
        edgeFnDownloadURL && (this.sandbox.edgeFnDownloadURL = edgeFnDownloadURL);
        this.network.networkSignalsFilterList.disallowed.addURLLike(...disallowListURLs.filter((val) => Boolean(val)));
        if (sampleRate && Math.random() <= sampleRate) {
            this.sampleSuccess = true;
        }
    }
}
class SignalsDebugSettings {
    constructor(disableRedaction, enableIngestion) {
        if (typeof disableRedaction === 'boolean') {
            this.setDebugKey(SignalsDebugSettings.redactionKey, disableRedaction);
        }
        if (typeof enableIngestion === 'boolean') {
            this.setDebugKey(SignalsDebugSettings.ingestionKey, enableIngestion);
        }
        // setting ?segment_signals_debug=true will disable redaction, enable ingestion, and set keys in local storage
        // this setting will persist across page loads (even if there is no query string)
        // in order to clear the setting, user must set ?segment_signals_debug=false
        const debugModeInQs = parseDebugModeQueryString();
        logger.debug('debugMode is set to true via query string');
        if (typeof debugModeInQs === 'boolean') {
            this.setDebugKey(SignalsDebugSettings.redactionKey, debugModeInQs);
            this.setDebugKey(SignalsDebugSettings.ingestionKey, debugModeInQs);
        }
    }
    setDebugKey(key, enable) {
        try {
            if (enable) {
                window.sessionStorage.setItem(key, 'true');
            }
            else {
                logger.debug(`Removing debug key ${key} from storage`);
                window.sessionStorage.removeItem(key);
            }
        }
        catch (e) {
            logger.debug('Storage error', e);
        }
    }
    getDisableSignalsRedaction() {
        try {
            const isEnabled = Boolean(window.sessionStorage.getItem(SignalsDebugSettings.redactionKey));
            if (isEnabled) {
                logger.debug(`${SignalsDebugSettings.redactionKey}=true (app. storage)`);
                return true;
            }
        }
        catch (e) {
            logger.debug('Storage error', e);
        }
        return false;
    }
    getEnableSignalsIngestion() {
        try {
            const isEnabled = Boolean(window.sessionStorage.getItem(SignalsDebugSettings.ingestionKey));
            if (isEnabled) {
                logger.debug(`${SignalsDebugSettings.ingestionKey}=true (app. storage)`);
                return true;
            }
        }
        catch (e) {
            logger.debug('Storage error', e);
        }
        return false;
    }
}
SignalsDebugSettings.redactionKey = 'segment_signals_debug_redaction_disabled';
SignalsDebugSettings.ingestionKey = 'segment_signals_debug_ingestion_enabled';
//# sourceMappingURL=settings.js.map